WITH BasePackage(Id, Name, InstallType, IsLocked, IsChanged, [Version], Maintainer) AS (
	SELECT Id, Name, InstallType, IsLocked, IsChanged, [Version], Maintainer
	FROM SysPackage WITH(NOLOCK)
	WHERE SysPackage.SysWorkspaceId in (select Id from SysWorkspace where Name = 'Default')
	AND SysPackage.Name <> 'Custom' AND SysPackage.Maintainer = 'Terrasoft'
),
UnlockCustomerPackagesSchemasCount(Id, Name, Count) AS (
	SELECT BasePackage.Id, BasePackage.Name, COUNT(BasePackage.Id) FROM SysSchema SysSchema WITH(NOLOCK)
	INNER JOIN BasePackage ON BasePackage.Id = SysSchema.SysPackageId
	WHERE InstallType = 0  
	AND (SysSchema.IsLocked = 1 AND SysSchema.IsChanged = 1)
	GROUP BY BasePackage.Id, BasePackage.Name
),
AllCustomerPackagesSchemasCount(Id, Name, Count) AS (
	SELECT BasePackage.Id, BasePackage.Name, COUNT(BasePackage.Id) FROM SysSchema SysSchema WITH(NOLOCK)
	INNER JOIN BasePackage ON BasePackage.Id = SysSchema.SysPackageId
	WHERE InstallType = 0  
	GROUP BY BasePackage.Id, BasePackage.Name
),
CustomerPackages(Id, Name) AS (
	SELECT BasePackage.Id, BasePackage.Name FROM SysSchema SysSchema WITH(NOLOCK)
	INNER JOIN BasePackage ON BasePackage.Id = SysSchema.SysPackageId
	INNER JOIN AllCustomerPackagesSchemasCount ON BasePackage.Id = AllCustomerPackagesSchemasCount.Id
	INNER JOIN UnlockCustomerPackagesSchemasCount ON BasePackage.Id = UnlockCustomerPackagesSchemasCount.Id
	WHERE InstallType = 0  
	AND UnlockCustomerPackagesSchemasCount.Count > 0
	AND UnlockCustomerPackagesSchemasCount.Count = AllCustomerPackagesSchemasCount.Count
	GROUP BY BasePackage.Id, BasePackage.Name
),
UnlockPackage(Id, Name, IsLocked, IsChanged, [Version], Maintainer, InstallType) AS (
	SELECT BasePackage.Id, BasePackage.Name, BasePackage.IsLocked, BasePackage.IsChanged, [Version], Maintainer, InstallType FROM BasePackage
	LEFT JOIN CustomerPackages ON CustomerPackages.Id = BasePackage.Id
	WHERE InstallType = 0 AND CustomerPackages.Id IS NULL
),
UnlockSchema(Id, Name, PackageName, IsLocked, IsChanged) AS (
	SELECT SysSchema.Id, SysSchema.Name, BasePackage.Name, SysSchema.IsLocked, SysSchema.IsChanged FROM SysSchema WITH(NOLOCK)
	INNER JOIN BasePackage ON BasePackage.Id = SysSchema.SysPackageId
	LEFT JOIN CustomerPackages ON CustomerPackages.Id = BasePackage.Id
	WHERE SysSchema.IsLocked = 1 AND SysSchema.IsChanged = 1
	--AND SysSchema.ManagerName <> 'ProcessSchemaManager'
	AND CustomerPackages.Id IS NULL
)
--      
SELECT
	DB_NAME() AS DBName,
	Name AS PackageName,
	null AS SchemaName,
	[Version] AS VersionPack,
	Maintainer AS Maintainer,
	IsLocked,
	IsChanged,
	InstallType AS InstallType,
	Id
INTO #UnlockSchemaAndPackages
FROM UnlockPackage
UNION ALL
SELECT 
	DB_NAME() AS DBName,
	PackageName AS PackageName,
	Name AS SchemaName,
	null AS VersionPack,
	null AS Maintainer,
	IsLocked,
	IsChanged,
	null AS InstallType,
	Id
FROM UnlockSchema;

--     
UPDATE [SysSchema]
SET
	ModifiedOn = CAST(-53690 AS DATETIME),
	MetaDataModifiedOn = cast(-53690 AS DATETIME)
WHERE [Id] IN (SELECT ss.Id FROM SysSchema ss JOIN SysPackage sp ON ss.SysPackageId = sp.Id WHERE sp.Name = 'NUI' AND ss.Name = 'ConfigurationSectionHelper');

UPDATE [SysPackage]
SET
	ModifiedOn = CAST(-53690 AS DATETIME)
WHERE Name = 'NUI';

--  
UPDATE SysPackage
SET IsChanged = 0, IsLocked = 0, InstallType = 1
WHERE Id IN (SELECT Id FROM #UnlockSchemaAndPackages WHERE SchemaName IS NULL);

--  
UPDATE SysSchema
SET IsChanged = 0, IsLocked = 0
WHERE Id IN (SELECT Id FROM #UnlockSchemaAndPackages WHERE InstallType IS NULL);

DROP TABLE #UnlockSchemaAndPackages;